from importlib import import_module
from itertools import chain
import requests
import logging

regions = [
    'Auvergne-Rhône-Alpes',
    'Bretagne',
    'Corse',
    'Pays de la Loire',
    'Centre-Val de Loire',
    'Nouvelle-Aquitaine',
    'Hauts-de-France',
    'Grand Est',
    'Occitanie',
    'Île-de-France',
    'Sud',
    'Normandie'
]

def insee_list():
    return list(chain(*[
        import_module(f'.{region_name}', 'indice_pollution.regions').Forecast().insee_list
        for region_name in regions
    ]))

def region(insee):
    try:
        insee = f'{int(insee):05}'
    except ValueError:
        pass

    r = requests.get(f'https://geo.api.gouv.fr/communes/{insee}', params={"fields": "region"})
    r.raise_for_status()
    region_name = r.json()['region']['nom']
    try:
        region = import_module(f'.{region_name}', 'indice_pollution.regions').Forecast()
    except ModuleNotFoundError as e:
        logging.error(f'Region {region_name} not found INSEE: {insee}')
        logging.error(e)
        raise KeyError

    return region


from flask import Flask
from flask_manage_webpack import FlaskManageWebpack
from flask_cors import CORS
from flask_migrate import Migrate
from datetime import datetime
import pytz
import os

def create_app(test_config=None):
    app = Flask(
        __name__,
        instance_relative_config=True,
        static_url_path=''
    )
    app.config.from_mapping(
        SECRET_KEY=os.getenv('SECRET_KEY', 'dev'),
        SQLALCHEMY_DATABASE_URI=os.getenv('SQLALCHEMY_DATABASE_URI') or os.getenv('POSTGRESQL_ADDON_URI'),
        SQLALCHEMY_TRACK_MODIFICATIONS=False
    )
    CORS(app, send_wildcard=True)

    manage_webpack = FlaskManageWebpack()
    manage_webpack.init_app(app)

    from .models import db
    db.init_app(app)
    migrate = Migrate(app, db)

    with app.app_context():
        import indice_pollution.api
        import indice_pollution.web
        import indice_pollution.history

    return app

def forecast(insee, date_=None, force_from_db=False):
    from .regions.solvers import region
    date_ = date_ or today()
    r = region(insee)
    return {
        "data": r.get(date_=date_, insee=insee, force_from_db=force_from_db),
        "metadata": {
            "region": {
                "nom": r.__module__.split(".")[-1],
                "website": r.website
            }
        }
    }

def today():
    zone = pytz.timezone('Europe/Paris')
    return datetime.now(tz=zone).date()


import sib_api_v3_sdk
from flask import _app_ctx_stack

class SendInBlue(object):
    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.configuration = sib_api_v3_sdk.Configuration()
        self.configuration['apikey'] = app.config['SIB_APIKEY']
        app.teardown_context(self.teardown)

    def teardown(self):
        ctx = _app_ctx_stack.top
        if hasattr(ctx, 'sendinblue'):
            ctx.sqlite3_db.close()

    @property
    def api(self):
        ctx = _app_ctx_stack.top
        if ctx is not None:
            if not hasattr(ctx, 'sendinblue'):
                apiclient = sib_api_v3_sdk.ApiClient(self.configuration)
                ctx.sendinblue = sib_api_v3_sdk.AccountApi(apiclient)
            return ctx.sendinblue




from datetime import timedelta
from . import ForecastMixin
import requests
from bs4 import BeautifulSoup
from datetime import datetime

class Forecast(ForecastMixin):
    website = 'https://www.atmo-nouvelleaquitaine.org/'
    url = 'https://opendata.atmo-na.org/geoserver/ind_nouvelle_aquitaine_agglo/wfs'

    attributes_key = 'properties'
    use_dateutil_parser = True

    insee_list = [
        '33063', '79005', '16102', '64102', '64445', '19272', '87085', '24322', '40088',
        '17300', '16015', '79191', '87154', '86194', '19031', '64300', '23096'
    ]
    COLOR_TO_QUALIF = {
        "0": "tres_bon",
        "21": "bon",
        "31": "bon",
        "41": "bon",
        "51": "moyen",
        "61": "mediocre",
        "71": "mediocre",
        "81": "mediocre",
        "91": "mauvais",
        "100": "tres_mauvais"
    }

    def get_no_cache(self, date_, insee, attempts=0):
        r = requests.get(f'https://www.atmo-nouvelleaquitaine.org/monair/commune/{insee}')
        r.raise_for_status()
        soup = BeautifulSoup(r.text, 'html.parser')

        controls = soup.find_all('div', class_='day-controls')
        days = controls[0].find_all('a', class_='raster-control-link')

        return [
            {
                "date": str(datetime.fromtimestamp(int(day.attrs.get('data-rasterid'))).date()),
                "indice": int(int(day.attrs.get('data-index'))/10),
                "qualif": self.COLOR_TO_QUALIF[day.attrs.get('data-color')]
            }
            for day in days
        ]

    def get_close_insee(self, insee):
        return insee


from . import ForecastMixin
from datetime import datetime, timedelta
from dateutil.parser import parse
import requests
from bs4 import BeautifulSoup
from flask import current_app

class Forecast(ForecastMixin):
    website = 'https://www.airparif.asso.fr/'
    url = 'https://services8.arcgis.com/gtmasQsdfwbDAQSQ/arcgis/rest/services/ind_idf_agglo/FeatureServer/0/query'

    insee_list = ['75056']

    INDICE_TO_QUALIF = {
        0: "tres_bon",
        1: "tres_bon",
        2: "bon",
        3: "bon",
        4: "bon",
        5: "moyen",
        6: "mediocre",
        7: "mediocre",
        8: "mediocre",
        9: "mauvais",
        10: "tres_mauvais"
    }

    def where(self, date_, insee):
        return f"date_ech >= CURRENT_DATE - INTERVAL '2' DAY"
    
    def get_close_insee(self, insee):
        return insee

    def get(self, date_, insee, attempts=0, force_from_db=False):
        to_return = super().get(date_, insee, attempts, force_from_db)

        for r in to_return:
            if r['date'] == str(date_):
                return to_return
        else:
            return self.get_from_scraping(to_return, date_)

    def get_from_scraping(self, previous_results, date_):
        r = requests.get('https://www.airparif.asso.fr/')
        r.raise_for_status()
        soup = BeautifulSoup(r.text, 'html.parser')

        data =  soup.find_all('div', class_='indices_data')[0].find_all('div')
        indice = int(data[1].text)
        qualif = self.INDICE_TO_QUALIF[indice]

        return previous_results + [
            {"date": str(date_), "valeur": indice, "indice": indice, "qualif": qualif}
        ]


from indice_pollution.models import db
import json

class IndiceHistory(db.Model):
    __table_args__ = {"schema": "indice_schema"}

    id = db.Column(db.Integer, primary_key=True)
    date_ = db.Column(db.Date, nullable=False)
    insee = db.Column(db.String, nullable=False)
    _features = db.Column("features", db.String, nullable=False)

    @property
    def features(self):
        return json.loads(self._features)

    @features.setter
    def features(self, value):
        self._features = json.dumps(value)

    @classmethod
    def get(cls, date_, insee):
        return cls.query.filter_by(date_=date_, insee=insee).first()

    @classmethod
    def get_after(cls, date_, insee):
        return [v.features for v in cls.query.filter(cls.date_ >= date_, cls.insee==insee).all()]

    @classmethod
    def get_or_create(cls, date_, insee):
        return  cls.get(date_, insee) or cls(date_=date_, insee=insee)


from . import ForecastMixin

class Forecast(ForecastMixin):
    website = 'http://www.atmo-grandest.eu/'
    url = 'https://services3.arcgis.com/Is0UwT37raQYl9Jj/arcgis/rest/services/ind_Grand_Est_commune_3j/FeatureServer/0/query'

    insee_list = [
        '8105', '57463', '67180', '67482', '88160', '57672', '10387', '68224', '68297',
        '57227', '68066', '52448', '51454', '54395', '51108'
    ]


from . import ForecastMixin

class Forecast(ForecastMixin):
    website =  'https://www.qualitaircorse.org/'
    url = 'https://services9.arcgis.com/VQopoXNvUqHYZHjY/arcgis/rest/services/ind_atmo_corse/FeatureServer/0/query'

    insee_list = ['2B096', '2B033', '2A004']


from . import ForecastMixin
from dateutil.parser import parse
from json.decoder import JSONDecodeError
import orjson as json
from itertools import takewhile
from string import printable

class Forecast(ForecastMixin):
    website = 'http://www.atmonormandie.fr/'
    url = 'https://dservices7.arcgis.com/FPRT1cIkPKcq73uN/arcgis/services/ind_normandie_agglo/WFSServer?service=wfs&request=getcapabilities'

    attributes_key = 'properties'
    use_dateutil_parser = True

    insee_list = ['76351', '14366', '76540', '14118', '50502', '27229', '50129', '61001']

    @classmethod
    def params(cls, date_, insee):
        filter_zone = f'<PropertyIsEqualTo><PropertyName>code_zone</PropertyName><Literal>{insee}</Literal></PropertyIsEqualTo>'
        filter_date = f'<PropertyIsGreaterThanOrEqualTo><PropertyName>date_ech</PropertyName><Literal>{date_}T00:00:00.000Z</Literal></PropertyIsGreaterThanOrEqualTo>'
        return {
            'service': 'wfs',
            'request': 'getfeature',
            'typeName': f'ind_normandie_agglo:ind_normandie_agglo',
            'Filter': f"<Filter><And>{filter_zone}{filter_date}</And></Filter>",
            'outputFormat': 'geojson',
        }

    @classmethod
    def features(cls, r):
        r.encoding = 'utf8'
        try:
            return r.json()['features']
        except JSONDecodeError:
            set_printable = set(printable + 'éèàçôêùà')
            clean_string = str("".join(takewhile(lambda c: c in set_printable, r.text)))
            return json.loads(clean_string)['features']




from . import ForecastMixin
import csv
from dateutil.parser import parse
from bs4 import BeautifulSoup
import requests
from datetime import date, timedelta

class Forecast(ForecastMixin):
    website = 'https://www.atmo-bfc.org/'
    url = 'https://www.atmo-bfc.org/medias/ajax/me_gateway.php'

    def get_one_attempt(self, url, params, attempts=0):
        r = requests.post(url,
            headers={
                'Accept-Encoding': '*',
                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
            },
            data=params
        )
        r.raise_for_status()
        return r

    def params(self, date_, insee):
        self.date_ = date_
        return {
            "meC": "abo_communes",
            "meM": "traitementAjax",
            "meIdC": "7448b8180480a8749b31941011165d92.MTQxNQ%3D%3D",
            "meT": "i",
            "paramsM[mode]": "refreshCartePolluant",
            "paramsM[meCibleId]": "7448b8180480a8749b31941011165d92_MTQxNQ_3D_3D",
            "paramsM[meDate]": date_,
            "paramsM[codeCommune]": insee,
            "paramsM[idPolluant]": -1,
            "paramsM[codeAgglo]": 3,
            "paramsM[changeJour]": 1,
            "paramsM[prevPolluant]": 0,
            "m_idPage": 183,
            "m_initialesLangue": "fr",
            "ajax": 1
        }

    def get_no_cache(self, date_, insee, attempts=0):
        if insee not in self.insee_list:
            insee = self.get_close_insee(insee)
        day_before = str((date_ - timedelta(days=1)))
        day_after = str((date_ + timedelta(days=1)))
        features_daybefore = self.get_multiple_attempts(self.url, self.params(day_before, insee))
        features_date = self.get_multiple_attempts(self.url, self.params(str(date_), insee))
        features_dayafter = self.get_multiple_attempts(self.url, self.params(day_after, insee))
        return list(
            filter(
                lambda s: s is not None,
                map(
                    self.getter,
                    [
                        features_daybefore,
                        features_date,
                        features_dayafter
                    ]
                )
            )
        )

    def features(self, r):
        soup = BeautifulSoup(r.json()["indices"], features="html5lib")
        valeurIndice = soup.find_all("div", class_="valeurIndice")[0]
        return {
            "indice": valeurIndice.find_all("span")[1].text,
            "date": self.date_
        }

    def getter(self, feature):
        return feature

    def get_close_insee(self, insee):
        return insee


from . import ForecastMixin
from datetime import timedelta, datetime

class Forecast(ForecastMixin):
    website = 'https://www.atmosud.org/'
    url = 'https://geoservices.atmosud.org/geoserver/ind_sudpaca/ows?service=WFS&version=1.1.0'
    fr_date_format = '%d-%m-%Y 00:00:00'

    insee_list = ['06029', '06088', '13001', '13055', '83137', '84007']

    attributes_key = 'properties'
    use_dateutil_parser = True

    @classmethod
    def params(cls, date_, insee):
        tomorrow_date = date_ + timedelta(days=1)

        fr_date = date_.strftime(cls.fr_date_format)
        fr_tomorrow = tomorrow_date.strftime(cls.fr_date_format)

        return {
            'service': 'WFS',
            'version': '1.1.0',
            'request': 'GetFeature',
            'typeName': 'ind_sudpaca:ind_sudpaca_agglo',
            'CQL_FILTER': f"code_zone='{insee}' AND (date_ech='{fr_date}' OR date_ech='{fr_tomorrow}')",
            'outputFormat': 'json'
        }
    def date_getter(self, attributes):
        return datetime.strptime(attributes['date_ech'], self.fr_date_format)
